<?php
/**************************************************************
*  Copyright notice
*
*  (c) 2010 Andreas Becker - websedit AG <extensions@websedit.de>
*
*  All rights reserved
*
*  This script is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; version 2 of the License.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/
/**
 * Based on the extension 'm1_google_analytics'
 * m1_google_analytics written by:
 * Dimitri Tarassenko (mitka@mitka.us),
 * Bjoern Kraus (kraus@phoenixwebsolutions.de)
 */

require_once (PATH_tslib.'class.tslib_pibase.php');
require_once (PATH_tslib.'class.tslib_content.php');

/**
 * This extension is used to insert the Google Analytics
 * Tracking code into your website
 *
 * @author Andreas Becker - websedit AG <extensions@websedit.de>
 *
 */
class tx_wegoogleanalytics extends tslib_pibase {

	public $prefixId = 'tx_wegoogleanalytics';
	public $scriptRelPath = 'tx_wegoogleanalytics.php';
	public $extKey = 'we_google_analytics';

	/**
	 * Insert Google Analytics Code before the output of the page
	 * 
	 * @param str &$content Pagecontent
	 * @param str $conf     TyposcriptConfig
	 * @return Void
	 */
	public function main ( &$content, $conf ) {
		$this->content = &$content;
        $this->conf = $conf;

		if (!$this->conf['account']) {
			return;
		}

		$content = $this->process($this->content);
	}

	/**
	 * Processes the configuration given by TypoScript
	 *
	 * @param str $con Pagecontent
	 * @return str Pagecontent
	 */
	protected function process( $con ) {
		// validate given account number
		if (preg_match('#^\b(UA|MO)-\d{4,10}-\d{1,4}\b$#i', $this->conf['account'])) {
			$accountCheckPassed = 1;
		} else {
			$accountCheckPassed = 0;
		}
		$content = $con;
		if ($accountCheckPassed) {
			switch($this->conf['type']){
				case 'mobile':
					$content = $this->insertMobileGaCode($con);
					break;
				case 'sync':
					$content = $this->insertSyncGaCode($con);
					break;
				case 'async':
				default:
					$content = $this->insertAsyncGaCode($con);
					// Async is default
			}
		} else {
			$errorMessage = '<!--'.chr(10);
			$errorMessage .= '     Ooops: Syntaxcheck of Google Analytics Account Number failed!'.chr(10);
			$errorMessage .= '     Maybe misspelled entry in config.tx_we_google_analytics.account.'.chr(10);
			$errorMessage .= '     You used '.htmlspecialchars($this->conf['account']).chr(10);
			$errorMessage .= '     Please use the following format UA-xxxx-y ,'.chr(10);
			$errorMessage .= '     or for mobile tracking, use MO-xxxx-y.'.chr(10);
			$errorMessage .= '-->'.chr(10);
			$content = $this->insertTrackerCode($con, $errorMessage, 'headEnd');
		}
		return $content;
	}

	/**
	 * Google Analytics Mobile Tracking Code (extended)
	 * 
	 * @param bool $ano Anonymize IP (1) or not (0)
	 * @param str  $acc Google Analytics Account Number
	 * @return str TrackerUrl
	 *
	 */
	protected function googleAnalyticsGetImageUrl($ano) {
		// Copyright 2009 Google Inc. All Rights Reserved.
		$url = '';
		$url .= t3lib_extMgm::extRelPath($this->extKey).'ga.php?';
		$url .= 'utmac=' . $this->conf['account'];
		$url .= '&utmn=' . rand(0, 0x7fffffff);
		$referer = t3lib_div::GPvar('HTTP_REFERER');
		$query = t3lib_div::GPvar('QUERY_STRING');
		$path = t3lib_div::GPvar('REQUEST_URI');
		if (empty($referer)) {
		  $referer = '-';
		}
		$url .= '&utmr=' . urlencode($referer);
		if (!empty($path)) {
		  $url .= '&utmp=' . urlencode($path);
		}
		$url .= '&guid=ON';
		// Extended to implement anonymizeIP function
		if ($ano) {
			$url .= '&ano='.$ano;
		}
		return str_replace('&', '&amp;', $url);
	}

	/**
	 * Google Analytics Mobile Tracking Code
	 * 
	 * @param str $con Pagecontent
	 * @return str Pagecontent
	 * 
	 */
	protected function insertMobileGaCode($con) {
		$anonym = ($this->conf['anonymized'] == 1) ? 1 : 0;
		$gaMobileAfterContent = '<img src="'. $this->googleAnalyticsGetImageUrl($anonym).'" alt="" />';
		$content = $this->insertTrackerCode($con, $gaMobileAfterContent, 'bodyEnd');
		return $content;
	}

	/**
	 * Google Analytics sync (Traditional)
	 * 
	 * @param str $con Pagecontent
	 * @return str Pagecontent
	 * 
	 */
	protected function insertSyncGaCode($con) {
		if ($this->conf['anonymized'] == 1) {
			$anonymizeIp = '_gat._anonymizeIp();';
		} else {
			$anonymizeIp = '';
		}

		/* Remove _gaq. and _gat. if set
		 * also remove configs without underscore (e.g. account) */
		$gaConf = array();
		foreach ($this->conf as $param => $val) {
			$param = htmlspecialchars($param);
			if (is_array($val)) {
				foreach ($val as $paramTwo => $valTwo) {
					$paramTwo = htmlspecialchars($paramTwo);
					$valTwo = htmlspecialchars($valTwo);
					if (substr($paramTwo, 0, 1) == '_') {
						$gaConf[$paramTwo] = $valTwo;
					}
				}
			} else {
				$val = htmlspecialchars($val);
				if (substr($param, 0, 1) == '_') {
					$gaConf[$param] = $val;
				}
			}
		}

		$options = '';
		foreach ($gaConf as $param => $val) {
			if ($val != 'true' && $val != 'false' && $val != '1' && $val != '0' && strpos($val, ',') === FALSE) {
				$val = '"'.$val.'"';
			}
			$val = str_replace('&amp;', '&', $val);
			$options .= 'pageTracker.'.$param.'('.$val.');'.chr(10);
		}

		$gaSyncBeforeContent = chr(10).'<script type="text/javascript">
/* <![CDATA[ */
 var gaJsHost = (("https:" == document.location.protocol) ? "https://ssl." : "http://www.");
 document.write("\<script src=\'" + gaJsHost + "google-analytics.com/ga.js\' type=\'text/javascript\'>\<\/script>" );
/* ]]> */
</script>
<script type="text/javascript">
/* <![CDATA[ */
 var pageTracker = _gat._getTracker("'.$this->conf['account'].'");
 '.$anonymizeIp.'
 '.$options.'
 pageTracker._initData();
/* ]]> */
</script>';

		$gaSyncAfterContent = '<script type="text/javascript">
/* <![CDATA[ */
 pageTracker._trackPageview();
/* ]]> */
</script>';

		$con = $this->insertTrackerCode($con, $gaSyncBeforeContent, 'bodyBegin');
		$content = $this->insertTrackerCode($con, $gaSyncAfterContent, 'bodyEnd');
		return $content;
	}

	/**
	 * Google Analytics async
	 * 
	 * @param str $con Pagecontent
	 * @return str Pagecontent
	 *
	 */
	protected function insertAsyncGaCode($con) {
		if ($this->conf['anonymized'] == 1) {
			$anonymizeIp = " ['_gat._anonymizeIp'],";
		} else {
			$anonymizeIp = '';
		}

		/* Remove _gaq. and _gat. if set
		 * also remove configs without underscore (e.g. account) */
		$gaConf = array();
		foreach ($this->conf as $param => $val) {
			$param = htmlspecialchars($param);
			if (is_array($val)) {
				foreach ($val as $paramTwo => $valTwo) {
					$paramTwo = htmlspecialchars($paramTwo);
					$valTwo = htmlspecialchars($valTwo);
					if (substr($paramTwo, 0, 1) == '_') {
						$gaConf[$paramTwo] = $valTwo;
					}
				}
			} else {
				$val = htmlspecialchars($val);
				if (substr($param, 0, 1) == '_') {
					$gaConf[$param] = $val;
				}
			}
		}

		$options = '';
		foreach ($gaConf as $param => $val) {
			if ($val != 'true' && $val != 'false' && $val != '1' && $val != '0' && strpos($val, ',') === FALSE) {
				$val = "'".$val."'";
			}
			$val = str_replace('&amp;', '&', $val);
			$options .= " ['".$param."', ".$val. "],";
		}

		$gaAsync .= '<script type="text/javascript">'.chr(10);
		$gaAsync .= '/* <![CDATA[ */'.chr(10);
		$gaAsync .= " var _gaq = [['_setAccount', '".$this->conf['account']."'],".$anonymizeIp.$options." ['_trackPageview']];".chr(10);
		$gaAsync .= " (function(d, t) {
  var g = d.createElement(t); g.async = true;
  g.src = ('https:' == d.location.protocol ? 'https://ssl' : 'http://www') + '.google-analytics.com/ga.js';
  var s = d.getElementsByTagName(t)[0]; s.parentNode.insertBefore(g, s);
 })(document, 'script');
/* ]]> */
</script>";

		$content = $this->insertTrackerCode($con, $gaAsync, 'headEnd');
		return $content;
	}

	/**
	 * Inserts the Tracker Code on the given position (headEnd, bodyBegin, bodyEnd)
	 *
	 * @param str $con      Pagecontent
	 * @param str $gaCode   Google Analytics Tracker Code
	 * @param str $position Position of the Tracker Code in the html document
	 * @return str          Pagecontent
	 */
	protected function insertTrackerCode($con, $gaCode, $position) {
		switch($position) {
			case 'headEnd':
				$content = str_replace('</head>', $gaCode.chr(10).'</head>', $con);
				break;
			case 'bodyBegin':
				$content = preg_replace('/(<body)([^>]*)>/', '\\1\\2>'.$gaCode.chr(10), $con);
				break;
			case 'bodyEnd':
				$content = str_replace('</body>', $gaCode.chr(10).'</body>', $con);
				break;
			default:
			// none
		}
		return $content;
	}
}

if (defined('TYPO3_MODE') && $TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/we_google_analytics/class.tx_wegoogleanalytics.php']) {
        include_once($TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/we_google_analytics/class.tx_wegoogleanalytics.php']);
}

?>